/*------------------------------------------------------------------------
 |  FILE            SpaceAudioEngine.java
 |  PART OF         SHH SpaceGame - a simple, networked multiplayer game.
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 +----------------------------------------------------------------------*/

package client;

import java.io.*;
import java.net.*;
import sun.audio.*;

import util.*;

/**
 *
 *
 * @author    Sverre H. Huseby
 *            &lt;<A HREF="mailto:shh@thathost.com">shh@thathost.com</A>&gt;
 * @version   $Id: SpaceAudioEngine.java,v 1.3 2002/01/27 14:47:32 sverrehu Exp $
 */
final class SpaceAudioEngine
extends AudioEngine {
/*-----------------------------------------------------------------------+
 |  PRIVATE PART                                                         |
 +----------------------------------------------------------------------*/
    private AudioData smallExplosion;
    private AudioData bigExplosion;
    private AudioData phaserHit;

    private AudioData readAudioResource(String resource) {
        URL         url;
        InputStream is;
        byte[]      buf;
        byte[]      buf2;
        int         len, q;

        if ((url = getClass().getResource(resource)) == null) {
            System.err.println("unable to get resource `" + resource + "'");
            return null;
        }
        buf = new byte[50000];
        try {
            is = url.openStream();
            len = is.read(buf);
            is.close();
        } catch (IOException e) {
            System.err.println(e.getMessage());
            return null;
        }
        buf2 = new byte[len];
        for (q = 0; q < len; q++)
            buf2[q] = buf[q];

        return new AudioData(buf2);
    }

/*-----------------------------------------------------------------------+
 |  PACKAGE LOCAL PART                                                   |
 +----------------------------------------------------------------------*/
    SpaceAudioEngine() {
        smallExplosion = readAudioResource("/audio/SmallExplosion.au");
        bigExplosion = readAudioResource("/audio/BigExplosion.au");
        phaserHit = readAudioResource("/audio/PhaserHit.au");
    }

    final void playSmallExplosion() {
        play(smallExplosion);
    }

    final void playBigExplosion() {
        play(bigExplosion);
    }

    final void playPhaserHit() {
        play(phaserHit);
    }
}
