/*------------------------------------------------------------------------
 |  FILE            AudioEngine.java
 |  PART OF         SHH SpaceGame - a simple, networked multiplayer game.
 |  WRITTEN BY      Sverre H. Huseby <shh@thathost.com>
 +----------------------------------------------------------------------*/

package util;

import java.io.*;
import java.util.*;
import sun.audio.*;

/**
 *
 *
 * @author    Sverre H. Huseby
 *            &lt;<A HREF="mailto:shh@thathost.com">shh@thathost.com</A>&gt;
 * @version   $Id: AudioEngine.java,v 1.3 2002/01/27 14:47:33 sverrehu Exp $
 */
public class AudioEngine
    implements Runnable {
/*-----------------------------------------------------------------------+
 |  PRIVATE PART                                                         |
 +----------------------------------------------------------------------*/
    private boolean enabled = true;
    private Thread  theThread;
    private Vector  whatToPlay;

/*-----------------------------------------------------------------------+
 |  INHERITANTS' INTERFACE                                               |
 +----------------------------------------------------------------------*/
    protected final void play(InputStream is) {
        if (!enabled || is == null || theThread == null)
            return;
        whatToPlay.addElement(is);
        theThread.resume();
    }

    protected final void play(AudioData ad) {
        if (!enabled || ad == null || theThread == null)
            return;
        whatToPlay.addElement(ad);
        theThread.resume();
    }

/*-----------------------------------------------------------------------+
 |  PUBLIC INTERFACE                                                     |
 +----------------------------------------------------------------------*/
    public AudioEngine() {
        whatToPlay = new Vector();
        theThread = new Thread(this);
        theThread.start();
    }

    public boolean isEnabled() {
        return enabled;
    }

    public void setEnabled(boolean onoff) {
        enabled = onoff;
    }

    /* Runnable *********************************************************/
    public void run() {
        Object what;

        for (;;) {
            theThread.suspend();
            while (!whatToPlay.isEmpty()) {
                what = whatToPlay.elementAt(0);
                whatToPlay.removeElementAt(0);
                try {
                    if (what instanceof AudioData)
                        AudioPlayer.player.start(
                            new AudioDataStream((AudioData) what));
                    else if (what instanceof InputStream)
                        AudioPlayer.player.start(
                            new AudioStream((InputStream) what));
                } catch (IOException e) {
                    System.out.println("Error in audio thread: "
                                       + e.getMessage());
                }
            }
        }
    }
}
